/*
 * conf.h
 *
 * This file is part of msmtp, an SMTP client.
 *
 * Copyright (C) 2000, 2003, 2004
 * Martin Lambers <marlam@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *   msmtp is released under the GPL with the additional exemption that
 *   compiling, linking, and/or using OpenSSL is allowed.
 */

#ifndef CONF_H
#define CONF_H

#include "list.h"

/* All 'errstr' arguments must point to buffers that are at least
 * 'errstr_bufsize' characters long.
 * If a function with an 'errstr' argument returns a value != CONF_EOK,
 * 'errstr' will contain an error message (maybe just "").
 * If such a function returns CONF_EOK, 'errstr' will not be changed.
 */
extern const size_t errstr_bufsize;
#define CONF_EOK		0	/* no error */
#define CONF_EIO		1	/* Input/output error */
#define CONF_EPARSE		2	/* Parse error */
#define CONF_ESYNTAX		3	/* Syntax error */

/*
 * An account
 */
typedef struct
{
    char *id;			/* name of this account */
    char *host;			/* hostname of SMTP server */
    int port;			/* port number */
    char *domain;		/* argument to EHLO command */
    char *from;			/* envelope from address */
    char *auth_mech;		/* authentication mechanism */
    char *username;		/* username for authentication */
    char *password;		/* password for authentication */
    char *ntlmdomain;		/* domain for NTLM authentication */
    int tls;			/* flag: use TLS? */
    char *tls_key_file;		/* file in PEM format */
    char *tls_cert_file;	/* file in PEM format */
    char *tls_trust_file;	/* file in PEM format */
    int tls_nocertcheck;	/* flag: do not check certificate? */
    int tls_nostarttls;		/* flag: start TLS immediatly (without STARTTLS command)? */
    char *dsn_return;		/* DSN return request */
    char *dsn_notify;		/* DSN notify request */
    int keepbcc;		/* flag: keep the Bcc header in the mail? */
    char *logfile;		/* logfile */
} account_t;

/*
 * You can remember which settings of an account you changed by using 
 * the following flags
 */
#define ACC_ID			1
#define ACC_HOST		2
#define ACC_PORT		4
#define ACC_DOMAIN		8
#define ACC_FROM		16
#define ACC_AUTH_MECH		32
#define ACC_USERNAME		64
#define ACC_PASSWORD		128
#define ACC_NTLMDOMAIN		256
#define ACC_TLS			512
#define ACC_TLS_KEY_FILE	1024
#define ACC_TLS_CERT_FILE	2048
#define ACC_TLS_TRUST_FILE	4096
#define ACC_TLS_NOCERTCHECK	8192
#define ACC_TLS_NOSTARTTLS	16384
#define ACC_DSN_RETURN		32768
#define ACC_DSN_NOTIFY		65536
#define ACC_KEEPBCC		131072
#define ACC_LOGFILE		262144

/* 
 * account_new()
 * 
 * Create a new account_t. Must be freed with account_free().
 */
account_t *account_new(void);

/* 
 * account_free()
 * 
 * Free an account_t.
 */
void account_free(void *a);

/*
 * find_account()
 * 
 * Find an account in a list of accounts by its id.
 * Returns the account or NULL.
 */
account_t *find_account(list_t *acc_list, const char *id);

/*
 * find_account_by_envelope_from()
 * 
 * Find an account in a list of accounts by its envelope from address.
 * Returns the *first* account that matches or NULL.
 */
account_t *find_account_by_envelope_from(list_t *acc_list, const char *from);

/*
 * is_on(), is_off()
 *
 * Check whether the given string is "on" or "off"
 */
int is_on(char *s);
int is_off(char *s);

/*
 * check_auth_arg()
 *
 * checks if the given string is a proper argument to the auth command.
 * If so, the string is converted to uppercase and 0 is returned. Otherwise, 1
 * is returned.
 * Note that you have to check whether the arg is "off" separately, because that
 * value results in NULL.
 */
int check_auth_arg(char *arg);

/*
 * check_dsn_notify_arg()
 *
 * checks if the given string is a proper argument to the dsn_notify command.
 * If so, the string is converted to uppercase and 0 is returned. Otherwise, 1
 * is returned.
 * Note that you have to check whether the arg is "off" separately, because that
 * value results in NULL.
 */
int check_dsn_notify_arg(char *arg);

/*
 * override_account()
 *
 * If a property flag is set in 'mask', override this property of 'acc1'
 * with the one from 'acc2'
 */
void override_account(account_t *acc1, account_t *acc2, int mask);

/* 
 * complete_account()
 * 
 * Complete an account_t (fill in default values for empty fields).
 */
void complete_account(account_t *acc);

/*
 * check_account()
 * 
 * Check an account_t.
 * Used error codes: CONF_ESYNTAX
 */
int check_account(account_t *acc, char *errstr);

/*
 * get_conf()
 * 
 * Read 'conffile' and store all account data in 'acc_list'.
 * Used error codes: CONF_EIO, CONF_EPARSE, CONF_ESYNTAX
 */
int get_conf(const char *conffile, list_t **acc_list, char *errstr);

#endif
