/*
 * paths.c
 *
 * This file is part of msmtp, an SMTP client.
 *
 * Copyright (C) 2004
 * Martin Lambers <marlam@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *   msmtp is released under the GPL with the additional exemption that
 *   compiling, linking, and/or using OpenSSL is allowed.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
extern int errno;

#ifdef WINDOWS
#include <windows.h>
#include <io.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#elif defined DJGPP
#include <unistd.h>
#else /* UNIX */
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#include <sys/types.h>
#include <sys/stat.h>
#include <pwd.h>
#endif

#include "xmalloc.h"
#include "paths.h"


#ifdef WINDOWS
#define PATH_SEP '\\'
#else /* UNIX or DJGPP */
#define PATH_SEP '/'
#endif


/*
 * get_prgname()
 *
 * see paths.h
 */

char *get_prgname(char *argv0)
{
    char *prgname;
    
    prgname = strrchr(argv0, PATH_SEP);
    if (!prgname)
    {
	prgname = argv0;
    }
    else
    {
	prgname++;
    }
    
    return prgname;
}


/*
 * get_homedir()
 *
 * Get the users home directory (or something similar, depending on the OS).
 * Returns a pointer to an allocated string in 'h'.
 * Used error codes: PATHS_ENOHOME
 */

int get_homedir(char **h, char *errstr)
{
#ifdef WINDOWS

    BYTE home[MAX_PATH + 1];
    HKEY hkey;
    DWORD len;
    DWORD type;
    long l;
    
    l = RegOpenKeyEx(HKEY_CURRENT_USER,
	    "Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders", 
	    0, KEY_READ, &hkey);
    if (l != ERROR_SUCCESS)
    {
	snprintf(errstr, errstr_bufsize, "cannot read registry");
	return PATHS_ENOHOME;
    }
    len = MAX_PATH;
    l = RegQueryValueEx(hkey, "AppData", NULL, &type, home, &len);
    if (l != ERROR_SUCCESS || len >= MAX_PATH)
    {
	RegCloseKey(hkey);
	snprintf(errstr, errstr_bufsize, "cannot read registry");
	return PATHS_ENOHOME;
    }
    RegCloseKey(hkey);
    *h = xstrdup((char *)home);
    return PATHS_EOK;

#elif defined DJGPP
    
    char *home;
    
    if (!(home = getenv("HOME")))
    {
	snprintf(errstr, errstr_bufsize, 
		"no environment variable HOME");
	return PATHS_ENOHOME;
    }
    *h = xstrdup(home);
    return PATHS_EOK;

#else /* UNIX */

    char *home;
    struct passwd *pw;
    
    if (!(home = getenv("HOME")))
    {
	pw = getpwuid(getuid());
	if (!pw || !(pw->pw_dir))
	{
	    snprintf(errstr, errstr_bufsize, 
		    "cannot get user's home directory");
	    return PATHS_ENOHOME;
	}
	home = pw->pw_dir;
    }
    *h = xstrdup(home);
    return PATHS_EOK;

#endif
}


/*
 * get_default_conffile()
 *
 * see paths.h
 */

int get_default_conffile(const char *base, char **dst, char *errstr)
{
    int e;
    char *conffile;
    size_t len;
    
    if ((e = get_homedir(&conffile, errstr)) != PATHS_EOK)
    {
	return e;
    }
    len = strlen(conffile);
    conffile = xrealloc(conffile, (len + strlen(base) + 2) * sizeof(char));
    if (len > 0 && conffile[len - 1] != PATH_SEP)
    {
	conffile[len++] = PATH_SEP;
    }
    strcpy(conffile + len, base);

    *dst = conffile;
    return PATHS_EOK;
}


/*
 * expand_tilde()
 *
 * see paths.h
 */

int expand_tilde(const char *filename, char **new_filename, char *errstr)
{
    int e;
    size_t homedirlen;
    
    if (filename[0] == '~')
    {
	if ((e = get_homedir(new_filename, errstr)) != PATHS_EOK)
	{
	    return e;
	}
	homedirlen = strlen(*new_filename);
	*new_filename = xrealloc(*new_filename, 
		(homedirlen + strlen(filename)) * sizeof(char));
	strcpy(*new_filename + homedirlen, filename + 1);
    }
    else 
    {
	*new_filename = xstrdup(filename);
    }
    return PATHS_EOK;
}


/*
 * [WINDOWS ONLY] mkstemp()
 *
 * Windows lacks mkstemp(). This function is a poor replacement.
 * The randomness of the unique filename is low, and there's only one attempt 
 * to safely open a file with this name.
 * However, it is still better than Windows' _mktemp() function.
 * Return value: file descriptor of the file or -1 on error, in which case errno
 * is set.
 */

#ifdef WINDOWS
int mkstemp(char *template)
{
    const char alnum[] = 
	"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
    size_t templatelen;
    char *p;
    int i;

    templatelen = strlen(template);
    if (templatelen < 6)
    {
	errno = EINVAL;
	return -1;
    }
    p = template + templatelen - 6;
    if (strcmp(p, "XXXXXX") != 0)
    {
	errno = EINVAL;
	return -1;
    }
    
    srand((unsigned int)time(NULL));
    for (i = 0; i < 6; i++)
    {
	p[i] = alnum[rand() % 62];
    }
    
    return _open(template, _O_CREAT | _O_EXCL, _S_IREAD | _S_IWRITE);
}
#endif


/*
 * tempfile()
 *
 * see paths.h
 */

int tempfile(const char *base, char **name, FILE **f, char *errstr)
{
    size_t baselen;
    const char *dir;
    size_t dirlen;
    char *template = NULL;
    size_t templatelen;
    char *template_bak = NULL;
    int fd = -1;
    int e;

    *name = NULL;

    /* the directory for the temp file */
    if (!(dir = getenv("TMPDIR")))
    {
	/* system dependent default location */
#ifdef WINDOWS
	/* there is no registry key for this */
	if (!(dir = getenv("TEMP")))
	{
	    if (!(dir = getenv("TMP")))
	    {
		dir = "C:";
	    }		    
	}
#elif defined DJGPP
	dir = "C:";
#else /* UNIX or whatever */
#ifdef P_tmpdir
	dir = P_tmpdir;
#else
	dir = "/tmp";
#endif
#endif
    }    
    dirlen = strlen(dir);

    /* the proposed file name */
    baselen = strlen(base);
#ifdef DJGPP
    /* shorten the base to two characters because of 8.3 filenames */
    if (baselen > 2)
    {
	baselen = 2;
    }
#endif
    
    /* build the template */
    templatelen = dirlen + 1 + baselen + 6;
    template = xmalloc((templatelen + 1) * sizeof(char));
    template_bak = xmalloc((templatelen + 1) * sizeof(char));
    strncpy(template, dir, dirlen);
    template[dirlen] = PATH_SEP;
    /* template is long enough */
    strcpy(template + dirlen + 1, base);
    strcpy(template + dirlen + 1 + baselen, "XXXXXX");
    strcpy(template_bak, template);

    /* create the file */
    if ((fd = mkstemp(template)) == -1)
    {
	/* we need template_bak because the contents of template are undefined
	 * when certain errors occur */
	snprintf(errstr, errstr_bufsize,
		"cannot create temporary file from template %s: %s",
		template_bak, strerror(errno));
	e = PATHS_ECANTCREAT;
	goto error_exit;
    }
    free(template_bak);
    template_bak = NULL;
    *name = template;

    /* set the rights (not every mkstemp() sets them to 0600) */
#ifndef DJGPP
#ifndef WINDOWS
    if (fchmod(fd, S_IRUSR | S_IWUSR) == -1)
    {
	snprintf(errstr, errstr_bufsize,
		"cannot set mode of temporary file %s: %s",
		*name, strerror(errno));
	e = PATHS_ECANTCREAT;
	goto error_exit;
    }
#endif /* not WINDOWS */
#endif /* not DJGPP */

    /* get the stream from the filedescriptor */
    if (!(*f = fdopen(fd, "w+")))
    {
	snprintf(errstr, errstr_bufsize,
		"cannot access temporary file %s: %s",
		*name, strerror(errno));
	e = PATHS_ECANTCREAT;
	goto error_exit;
    }

    return PATHS_EOK;

error_exit:
    if (fd >= 0)
    {
	close(fd);
    }
    if (*name)
    {
	(void)remove(*name);
    }
    free(template);
    free(template_bak);
    *f = NULL;
    *name = NULL;
    return e;
}
